@@ -14,9 +14,8 @@ __copyright__ = '''
 '''
 
 
-from dbusmock.mockobject import (DBusMockObject, MOCK_IFACE,
-                                 OBJECT_MANAGER_IFACE, get_object, get_objects)
-from dbusmock.testcase import DBusTestCase
+from dbusmock.mockobject import MOCK_IFACE, OBJECT_MANAGER_IFACE, DBusMockObject, get_object, get_objects
+from dbusmock.testcase import BusType, DBusTestCase, PrivateDBus, SpawnedMock
 
 try:
     # created by setuptools_scm
@@ -26,4 +25,4 @@ except ImportError:
 
 
 __all__ = ['DBusMockObject', 'MOCK_IFACE', 'OBJECT_MANAGER_IFACE',
-           'DBusTestCase', 'get_object', 'get_objects']
+           'DBusTestCase', 'PrivateDBus', 'BusType', 'SpawnedMock', 'get_object', 'get_objects']
