@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 '''bluetoothd mock template
 
 This creates the expected methods and properties of the object manager
@@ -16,9 +14,13 @@ This supports BlueZ 5 only.
 # of the license.
 
 __author__ = 'Philip Withnall'
-__copyright__ = '(c) 2013 Collabora Ltd.'
+__copyright__ = '''
+(c) 2013 Collabora Ltd.
+(c) 2017 - 2022 Martin Pitt <martin@piware.de>
+'''
+
+from pathlib import Path
 
-import os
 import dbus
 
 from dbusmock import OBJECT_MANAGER_IFACE, mockobject
@@ -36,6 +38,9 @@ MEDIA_IFACE = 'org.bluez.Media1'
 NETWORK_SERVER_IFACE = 'org.bluez.Network1'
 DEVICE_IFACE = 'org.bluez.Device1'
 
+# The device class of some arbitrary Android phone.
+MOCK_PHONE_CLASS = 5898764
+
 
 @dbus.service.method(AGENT_MANAGER_IFACE,
                      in_signature='os', out_signature='')
@@ -119,16 +124,16 @@ def RemoveDevice(adapter, path):
 @dbus.service.method(ADAPTER_IFACE,
                      in_signature='', out_signature='')
 def StartDiscovery(adapter):
-    adapter.discovering = True
+    adapter.props[ADAPTER_IFACE]['Discovering'] = True
     # NOTE: discovery filter support is minimal to mock
     # the Discoverable discovery filter
-    if adapter.discovery_filter is not None:
-        adapter.discoverable = True
+    if adapter.props[ADAPTER_IFACE]['DiscoveryFilter'] is not None:
+        adapter.props[ADAPTER_IFACE]['Discoverable'] = True
     adapter.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
         ADAPTER_IFACE,
         {
-            'Discoverable': dbus.Boolean(adapter.discoverable, variant_level=1),
-            'Discovering': dbus.Boolean(adapter.discovering, variant_level=1),
+            'Discoverable': dbus.Boolean(adapter.props[ADAPTER_IFACE]['Discoverable'], variant_level=1),
+            'Discovering': dbus.Boolean(adapter.props[ADAPTER_IFACE]['Discovering'], variant_level=1),
         },
         [],
     ])
@@ -137,16 +142,16 @@ def StartDiscovery(adapter):
 @dbus.service.method(ADAPTER_IFACE,
                      in_signature='', out_signature='')
 def StopDiscovery(adapter):
-    adapter.discovering = False
+    adapter.props[ADAPTER_IFACE]['Discovering'] = False
     # NOTE: discovery filter support is minimal to mock
     # the Discoverable discovery filter
-    if adapter.discovery_filter is not None:
-        adapter.discoverable = False
+    if adapter.props[ADAPTER_IFACE]['DiscoveryFilter'] is not None:
+        adapter.props[ADAPTER_IFACE]['Discoverable'] = False
     adapter.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
         ADAPTER_IFACE,
         {
-            'Discoverable': dbus.Boolean(adapter.discoverable, variant_level=1),
-            'Discovering': dbus.Boolean(adapter.discovering, variant_level=1),
+            'Discoverable': dbus.Boolean(adapter.props[ADAPTER_IFACE]['Discoverable'], variant_level=1),
+            'Discovering': dbus.Boolean(adapter.props[ADAPTER_IFACE]['Discovering'], variant_level=1),
         },
         [],
     ])
@@ -155,7 +160,7 @@ def StopDiscovery(adapter):
 @dbus.service.method(ADAPTER_IFACE,
                      in_signature='a{sv}', out_signature='')
 def SetDiscoveryFilter(adapter, discovery_filter):
-    adapter.discovery_filter = discovery_filter
+    adapter.props[ADAPTER_IFACE]['DiscoveryFilter'] = discovery_filter
 
 
 @dbus.service.method(BLUEZ_MOCK_IFACE,
@@ -170,8 +175,8 @@ def AddAdapter(self, device_name, system_name):
     '''
     path = '/org/bluez/' + device_name
     address_start = int(device_name[-1])
-    address = f"{address_start:02d}:{address_start+1:02d}:{address_start+2:02d}:" + \
-        f"{address_start+3:02d}:{address_start+4:02d}:{address_start+5:02d}"
+    address = (f"{address_start:02d}:{address_start + 1:02d}:{address_start + 2:02d}:"
+               f"{address_start + 3:02d}:{address_start + 4:02d}:{address_start + 5:02d}")
     adapter_properties = {
         'UUIDs': dbus.Array([
             # Reference:
@@ -217,9 +222,6 @@ def AddAdapter(self, device_name, system_name):
                    ])
 
     adapter = mockobject.objects[path]
-    adapter.discovering = False
-    adapter.discoverable = False
-    adapter.discovery_filter = None
     adapter.AddMethods(MEDIA_IFACE, [
         ('RegisterEndpoint', 'oa{sv}', '', ''),
         ('UnregisterEndpoint', 'o', '', ''),
@@ -259,6 +261,37 @@ def RemoveAdapter(self, device_name):
                        ])
 
 
+@dbus.service.method(BLUEZ_MOCK_IFACE,
+                     in_signature='s')
+def RemoveAdapterWithDevices(self, device_name):
+    '''Convenience method to remove a Bluetooth adapter and all
+       the devices associated to it
+    '''
+    adapter_path = '/org/bluez/' + device_name
+    adapter = mockobject.objects[adapter_path]
+    manager = mockobject.objects['/']
+
+    to_remove = []
+    for path in mockobject.objects:
+        if path.startswith(adapter_path + '/'):
+            to_remove.append(path)
+
+    for path in to_remove:
+        adapter.RemoveObject(path)
+        manager.EmitSignal(OBJECT_MANAGER_IFACE, 'InterfacesRemoved',
+                           'oas', [
+                               dbus.ObjectPath(path),
+                               [DEVICE_IFACE],
+                           ])
+
+    self.RemoveObject(adapter_path)
+    manager.EmitSignal(OBJECT_MANAGER_IFACE, 'InterfacesRemoved',
+                       'oas', [
+                           dbus.ObjectPath(adapter_path),
+                           [ADAPTER_IFACE],
+                       ])
+
+
 @dbus.service.method(DEVICE_IFACE,
                      in_signature='', out_signature='')
 def Pair(device):
@@ -267,8 +300,8 @@ def Pair(device):
             'Device already paired',
             name='org.bluez.Error.AlreadyExists')
     device_address = device.props[DEVICE_IFACE]['Address']
-    adapter_device_name = os.path.basename(device.props[DEVICE_IFACE]['Adapter'])
-    device.PairDevice(adapter_device_name, device_address)
+    adapter_device_name = Path(device.props[DEVICE_IFACE]['Adapter']).name
+    device.PairDevice(adapter_device_name, device_address, MOCK_PHONE_CLASS)
 
 
 @dbus.service.method(DEVICE_IFACE,
@@ -329,17 +362,29 @@ def AddDevice(self, adapter_device_name, device_address, alias):
             name=BLUEZ_MOCK_IFACE + '.NoSuchAdapter')
 
     properties = {
-        'UUIDs': dbus.Array([], signature='s', variant_level=1),
-        'Blocked': dbus.Boolean(False, variant_level=1),
-        'Connected': dbus.Boolean(False, variant_level=1),
-        'LegacyPairing': dbus.Boolean(False, variant_level=1),
-        'Paired': dbus.Boolean(False, variant_level=1),
-        'Trusted': dbus.Boolean(False, variant_level=1),
-        'RSSI': dbus.Int16(-79, variant_level=1),  # arbitrary
-        'Adapter': dbus.ObjectPath(adapter_path, variant_level=1),
         'Address': dbus.String(device_address, variant_level=1),
-        'Alias': dbus.String(alias, variant_level=1),
+        'AddressType': dbus.String('public', variant_level=1),
         'Name': dbus.String(alias, variant_level=1),
+        'Icon': dbus.String('', variant_level=1),
+        'Class': dbus.UInt32(0, variant_level=1),
+        'Appearance': dbus.UInt16(0, variant_level=1),
+        'UUIDs': dbus.Array([], signature='s', variant_level=1),
+        'Paired': dbus.Boolean(False, variant_level=1),
+        'Connected': dbus.Boolean(False, variant_level=1),
+        'Trusted': dbus.Boolean(False, variant_level=1),
+        'Blocked': dbus.Boolean(False, variant_level=1),
+        'WakeAllowed': dbus.Boolean(False, variant_level=1),
+        'Alias': dbus.String(alias, variant_level=1),
+        'Adapter': dbus.ObjectPath(adapter_path, variant_level=1),
+        'LegacyPairing': dbus.Boolean(False, variant_level=1),
+        'Modalias': dbus.String('', variant_level=1),
+        'RSSI': dbus.Int16(-79, variant_level=1),  # arbitrary
+        'TxPower': dbus.Int16(0, variant_level=1),
+        'ManufacturerData': dbus.Array([], signature='a{qv}', variant_level=1),
+        'ServiceData': dbus.Array([], signature='a{sv}', variant_level=1),
+        'ServicesResolved': dbus.Boolean(False, variant_level=1),
+        'AdvertisingFlags': dbus.Array([], signature='ay', variant_level=1),
+        'AdvertisingData': dbus.Array([], signature='a{yv}', variant_level=1),
     }
 
     self.AddObject(path,
@@ -371,7 +416,7 @@ def AddDevice(self, adapter_device_name, device_address, alias):
 
 @dbus.service.method(BLUEZ_MOCK_IFACE,
                      in_signature='ssi', out_signature='')
-def PairDevice(_self, adapter_device_name, device_address, class_=5898764):
+def PairDevice(_self, adapter_device_name, device_address, class_):
     '''Convenience method to mark an existing device as paired.
 
     You have to specify a device address which must be a valid Bluetooth
@@ -380,7 +425,7 @@ def PairDevice(_self, adapter_device_name, device_address, class_=5898764):
 
     This unblocks the device if it was blocked.
 
-    If the specified adapter or device don’t exist, a NoSuchAdapter or
+    If the specified adapter or device doesn't exist, a NoSuchAdapter or
     NoSuchDevice error will be returned on the bus.
 
     Returns nothing.
@@ -456,7 +501,7 @@ def BlockDevice(_self, adapter_device_name, device_address):
 
     This disconnects the device if it was connected.
 
-    If the specified adapter or device don’t exist, a NoSuchAdapter or
+    If the specified adapter or device doesn't exist, a NoSuchAdapter or
     NoSuchDevice error will be returned on the bus.
 
     Returns nothing.
@@ -499,7 +544,7 @@ def ConnectDevice(_self, adapter_device_name, device_address):
 
     This unblocks the device if it was blocked.
 
-    If the specified adapter or device don’t exist, a NoSuchAdapter or
+    If the specified adapter or device doesn't exist, a NoSuchAdapter or
     NoSuchDevice error will be returned on the bus.
 
     Returns nothing.
@@ -543,9 +588,9 @@ def DisconnectDevice(_self, adapter_device_name, device_address):
     address (e.g. 'AA:BB:CC:DD:EE:FF'). The adapter device name is the
     device_name passed to AddAdapter.
 
-    This does not change the device’s blocked status.
+    This does not change the device's blocked status.
 
-    If the specified adapter or device don’t exist, a NoSuchAdapter or
+    If the specified adapter or device doesn't exist, a NoSuchAdapter or
     NoSuchDevice error will be returned on the bus.
 
     Returns nothing.
