@@ -13,16 +13,18 @@ property such as 'NetworkingEnabled', or 'WirelessEnabled' etc. in
 # of the license.
 
 __author__ = 'Iftikhar Ahmad'
-__copyright__ = '(c) 2012 Canonical Ltd.'
+__copyright__ = '''
+(c) 2012 Canonical Ltd.
+(c) 2017 - 2022 Martin Pitt <martin@piware.de>
+'''
 
-import uuid
 import binascii
+import uuid
 
 import dbus
 
-from dbusmock import MOCK_IFACE
 import dbusmock
-
+from dbusmock import MOCK_IFACE
 
 BUS_NAME = 'org.freedesktop.NetworkManager'
 MAIN_OBJ = '/org/freedesktop'
@@ -165,9 +167,7 @@ def activate_connection(self, conn, dev, ap):
     devices = []
     if str(dev) != '/':
         devices.append(dev)
-    active_conn = dbus.ObjectPath(AddActiveConnection(self, devices, conn, ap, str(count), state))
-
-    return active_conn
+    return dbus.ObjectPath(AddActiveConnection(self, devices, conn, ap, str(count), state))
 
 
 def deactivate_connection(self, active_conn_path):
